// version 1.0
using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;

namespace PhoneInfo
{
    public class PhoneInfo
    {
        private static double version = 1.0;

        private string manufacturer;
        private string model;
        private string revision;
        private string imei;
        private string subscriberId;
        //original code location:
        //http://www.developersdex.com/vb/message.asp?p=2916&ID=%3C68D1F07B-ECB0-4A15-AFCA-2A911FC3234C%40microsoft.com%3E
        public void GetInfo()
        {
            IntPtr hLine;
            int dwNumDev;
            int num1 = 0x20000;
            _LINEINITIALIZEEXPARAMS lineInitializeParams = new _LINEINITIALIZEEXPARAMS();
            lineInitializeParams.dwTotalSize = (uint)Marshal.SizeOf(lineInitializeParams);
            lineInitializeParams.dwNeededSize = lineInitializeParams.dwTotalSize;
            lineInitializeParams.dwOptions = 2;
            lineInitializeParams.hEvent = IntPtr.Zero;
            lineInitializeParams.hCompletionPort = IntPtr.Zero;
            
            int result = lineInitializeEx(out hLine, IntPtr.Zero,
            IntPtr.Zero, null, out dwNumDev, ref num1, ref lineInitializeParams);
            if (result != 0)
            {
                throw new ApplicationException(string.Format("lineInitializeEx failed!\n\nError Code:{0}", result.ToString()));
            }
            int version;
            int dwAPIVersionLow = 0x10004;
            int dwAPIVersionHigh = 0x20000;
            _LINEEXTENSIONID lineExtensionID;
            result = lineNegotiateAPIVersion(hLine, 0, dwAPIVersionLow, dwAPIVersionHigh, out version, out lineExtensionID);
            if (result != 0)
            {
                throw new ApplicationException(string.Format("lineNegotiateAPIVersion failed!\n\nError Code: {0}", result.ToString()));
            }

            IntPtr hLine2 = IntPtr.Zero;
            result = lineOpen(hLine, 0, out hLine2, version, 0, IntPtr.Zero, 0x00000002, 0x00000004, IntPtr.Zero);
            if (result != 0)
            {
                throw new ApplicationException(string.Format("lineNegotiateAPIVersion failed!\n\nError Code: {0}", result.ToString()));
            }

            int structSize = Marshal.SizeOf(new _LINEGENERALINFO());
            byte[] bytes = new byte[structSize];
            byte[] tmpBytes = BitConverter.GetBytes(structSize);

            for (int index = 0; index < tmpBytes.Length; index++)
            {
                bytes[index] = tmpBytes[index];
            }

            result = lineGetGeneralInfo(hLine2, bytes);

            // get the needed size
            int neededSize = BitConverter.ToInt32(bytes, 4);

            // resize the array
            bytes = new byte[neededSize];

            // write out the new allocated size to the byte stream
            tmpBytes = BitConverter.GetBytes(neededSize);
            for (int index = 0; index < tmpBytes.Length; index++)
            {
                bytes[index] = tmpBytes[index];
            }

            // fetch the information with properly size buffer
            result = lineGetGeneralInfo(hLine2, bytes);

            if (result != 0)
            {
                throw new ApplicationException(Marshal.GetLastWin32Error().ToString());
            }


            int size;
            int offset;

            // manufacture
            size = BitConverter.ToInt32(bytes, 12);
            offset = BitConverter.ToInt32(bytes, 16);
            manufacturer = Encoding.Unicode.GetString(bytes, offset, size);
            manufacturer = manufacturer.Substring(0, manufacturer.IndexOf('\0'));

            // model
            size = BitConverter.ToInt32(bytes, 20);
            offset = BitConverter.ToInt32(bytes, 24);
            model = Encoding.Unicode.GetString(bytes, offset, size);
            model = model.Substring(0, model.IndexOf('\0'));

            // revision
            size = BitConverter.ToInt32(bytes, 28);
            offset = BitConverter.ToInt32(bytes, 32);
            revision = Encoding.Unicode.GetString(bytes, offset, size);
            revision = revision.Substring(0, revision.IndexOf('\0'));

            // serial number
            size = BitConverter.ToInt32(bytes, 36);
            offset = BitConverter.ToInt32(bytes, 40);
            imei = Encoding.Unicode.GetString(bytes, offset, size);
            imei = imei.Substring(0, imei.IndexOf('\0'));

            // subscriber id
            size = BitConverter.ToInt32(bytes, 44);
            offset = BitConverter.ToInt32(bytes, 48);
            subscriberId = Encoding.Unicode.GetString(bytes, offset, size);
            subscriberId = subscriberId.Substring(0, subscriberId.IndexOf('\0'));

            //tear down
            lineClose(hLine2);
            lineShutdown(hLine);

        }

        public double DllVersion
        { get { return version; } }

        // public properties
        public string Manufacturer
        { get { return manufacturer; } }

        public string Model
        { get { return model; } }

        public string Revision
        { get { return revision; } }

        public string SubscriberNumber
        { get { return subscriberId; } }

        public string Imei
        { get { return imei; } }


        // native stuff
        public class _LINEGENERALINFO
        {
            public int dwManufacturerOffset;
            public int dwManufacturerSize;
            public int dwModelOffset;
            public int dwModelSize;
            public int dwNeededSize;
            public int dwRevisionOffset;
            public int dwRevisionSize;
            public int dwSerialNumberOffset;
            public int dwSerialNumberSize;
            public int dwSubscriberNumberOffset;
            public int dwSubscriberNumberSize;
            public int dwTotalSize;
            public int dwUsedSize;
        }

        [StructLayout(LayoutKind.Sequential)]
        public struct _LINEEXTENSIONID
        {
            public IntPtr dwExtensionID0;
            public IntPtr dwExtensionID1;
            public IntPtr dwExtensionID2;
            public IntPtr dwExtensionID3;
        }

        [StructLayout(LayoutKind.Sequential)]
        public struct _LINEINITIALIZEEXPARAMS
        {
            public uint dwTotalSize;
            public uint dwNeededSize;
            public uint dwUsedSize;
            public uint dwOptions;
            public System.IntPtr hEvent;
            public System.IntPtr hCompletionPort;
            public uint dwCompletionKey;
        }

        // simple managed wrapper for the Telephony API
        [DllImport("coredll")]
        private static extern int lineInitializeEx(out IntPtr lpm_hLineApp, IntPtr hInstance, IntPtr lpfnCallback, string lpszFriendlyAppName, out int lpdwNumDevs, ref int lpdwAPIVersion, ref _LINEINITIALIZEEXPARAMS lpLineInitializeExParams);

        [DllImport("coredll")]
        private static extern int lineOpen(IntPtr m_hLineApp, int dwDeviceID, out IntPtr lphLine, int dwAPIVersion, int dwExtVersion, IntPtr dwCallbackInstance, int dwPrivileges, int dwMediaModes, IntPtr lpCallParams);

        [DllImport("coredll")]
        private static extern int lineNegotiateAPIVersion(IntPtr m_hLineApp, int dwDeviceID, int dwAPILowVersion, int dwAPIHighVersion, out int lpdwAPIVersion, out _LINEEXTENSIONID lpExtensionId);

        [DllImport("cellcore")]
        private static extern int lineGetGeneralInfo(IntPtr hLine, byte[] bytes);

        [DllImport("cellcore")]
        private static extern int lineGetGeneralInfo(IntPtr hLine, ref _LINEGENERALINFO lineGenerlInfo);

        [DllImport("coredll")]
        private static extern int lineClose(IntPtr hLine);

        [DllImport("coredll")]
        private static extern int lineShutdown(IntPtr m_hLineApp);



    }
}
